"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScheduleEntryLockCCBehaviors = void 0;
const cc_1 = require("@zwave-js/cc");
const ScheduleEntryLockCC_1 = require("@zwave-js/cc/ScheduleEntryLockCC");
const safe_1 = require("@zwave-js/core/safe");
const UserCode_1 = require("./UserCode");
const defaultCapabilities = {
    numWeekDaySlots: 1,
    numYearDaySlots: 0,
    numDailyRepeatingSlots: 0,
};
const STATE_KEY_PREFIX = "ScheduleEntryLock_";
const StateKeys = {
    standardOffset: `${STATE_KEY_PREFIX}standardOffset`,
    dstOffset: `${STATE_KEY_PREFIX}dstOffset`,
    schedule: (userId, slotId, kind) => `${STATE_KEY_PREFIX}schedule_${userId}_${slotId}_${kind}`,
};
const respondToScheduleEntryLockSupportedGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCSupportedGet) {
            const capabilities = {
                ...defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["Schedule Entry Lock"], receivedCC.endpointIndex),
            };
            const cc = new ScheduleEntryLockCC_1.ScheduleEntryLockCCSupportedReport(self.host, {
                nodeId: controller.host.ownNodeId,
                ...capabilities,
            });
            return { action: "sendCC", cc };
        }
    },
};
const respondToScheduleEntryLockTimeOffsetSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCTimeOffsetSet) {
            self.state.set(StateKeys.standardOffset, receivedCC.standardOffset);
            self.state.set(StateKeys.dstOffset, receivedCC.dstOffset);
            return { action: "ok" };
        }
    },
};
const respondToScheduleEntryLockTimeOffsetGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCTimeOffsetGet) {
            const cc = new ScheduleEntryLockCC_1.ScheduleEntryLockCCTimeOffsetReport(self.host, {
                nodeId: controller.host.ownNodeId,
                standardOffset: (self.state.get(StateKeys.standardOffset)
                    ?? 0),
                dstOffset: (self.state.get(StateKeys.dstOffset) ?? 0),
            });
            return { action: "sendCC", cc };
        }
    },
};
const respondToScheduleEntryLockEnableSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCEnableSet) {
            // No need to do anything, this cannot be queried
            return { action: "ok" };
        }
    },
};
const respondToScheduleEntryLockEnableAllSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCEnableAllSet) {
            // No need to do anything, this cannot be queried
            return { action: "ok" };
        }
    },
};
const respondToScheduleEntryLockWeekDayScheduleSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCWeekDayScheduleSet) {
            const userCodeCapabilities = {
                ...UserCode_1.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            // If the user identifier is out of range, the command will be ignored
            const userId = receivedCC.userId;
            if (userId > userCodeCapabilities.numUsers) {
                return { action: "fail" };
            }
            const capabilities = {
                ...defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["Schedule Entry Lock"], receivedCC.endpointIndex),
            };
            const slotId = receivedCC.slotId;
            // Ignore out of range slot queries
            if (slotId > capabilities.numWeekDaySlots) {
                return { action: "fail" };
            }
            const kind = cc_1.ScheduleEntryLockScheduleKind.WeekDay;
            const schedule = receivedCC.action === cc_1.ScheduleEntryLockSetAction.Set
                ? {
                    weekday: receivedCC.weekday,
                    startHour: receivedCC.startHour,
                    startMinute: receivedCC.startMinute,
                    stopHour: receivedCC.stopHour,
                    stopMinute: receivedCC.stopMinute,
                }
                : undefined;
            self.state.set(StateKeys.schedule(userId, slotId, kind), schedule);
            return { action: "ok" };
        }
    },
};
const respondToScheduleEntryLockWeekDayScheduleGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCWeekDayScheduleGet) {
            const userCodeCapabilities = {
                ...UserCode_1.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            // If the user identifier is out of range, the command will be ignored
            const userId = receivedCC.userId;
            if (userId > userCodeCapabilities.numUsers) {
                return { action: "fail" };
            }
            const capabilities = {
                ...defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["Schedule Entry Lock"], receivedCC.endpointIndex),
            };
            const slotId = receivedCC.slotId;
            // Ignore out of range slot queries
            if (slotId > capabilities.numWeekDaySlots) {
                return { action: "fail" };
            }
            const kind = cc_1.ScheduleEntryLockScheduleKind.WeekDay;
            const schedule = (self.state.get(StateKeys.schedule(userId, slotId, kind)) ?? {});
            const cc = new ScheduleEntryLockCC_1.ScheduleEntryLockCCWeekDayScheduleReport(self.host, {
                nodeId: controller.host.ownNodeId,
                userId,
                slotId,
                ...schedule,
            });
            return { action: "sendCC", cc };
        }
    },
};
const respondToScheduleEntryLockYearDayScheduleSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCYearDayScheduleSet) {
            const userCodeCapabilities = {
                ...UserCode_1.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            // If the user identifier is out of range, the command will be ignored
            const userId = receivedCC.userId;
            if (userId > userCodeCapabilities.numUsers) {
                return { action: "fail" };
            }
            const capabilities = {
                ...defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["Schedule Entry Lock"], receivedCC.endpointIndex),
            };
            const slotId = receivedCC.slotId;
            // Ignore out of range slot queries
            if (slotId > capabilities.numYearDaySlots) {
                return { action: "fail" };
            }
            const kind = cc_1.ScheduleEntryLockScheduleKind.YearDay;
            const schedule = receivedCC.action === cc_1.ScheduleEntryLockSetAction.Set
                ? {
                    startYear: receivedCC.startYear,
                    startMonth: receivedCC.startMonth,
                    startDay: receivedCC.startDay,
                    startHour: receivedCC.startHour,
                    startMinute: receivedCC.startMinute,
                    stopYear: receivedCC.stopYear,
                    stopMonth: receivedCC.stopMonth,
                    stopDay: receivedCC.stopDay,
                    stopHour: receivedCC.stopHour,
                    stopMinute: receivedCC.stopMinute,
                }
                : undefined;
            self.state.set(StateKeys.schedule(userId, slotId, kind), schedule);
            return { action: "ok" };
        }
    },
};
const respondToScheduleEntryLockYearDayScheduleGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCYearDayScheduleGet) {
            const userCodeCapabilities = {
                ...UserCode_1.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            // If the user identifier is out of range, the command will be ignored
            const userId = receivedCC.userId;
            if (userId > userCodeCapabilities.numUsers) {
                return { action: "fail" };
            }
            const capabilities = {
                ...defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["Schedule Entry Lock"], receivedCC.endpointIndex),
            };
            const slotId = receivedCC.slotId;
            // Ignore out of range slot queries
            if (slotId > capabilities.numYearDaySlots) {
                return { action: "fail" };
            }
            const kind = cc_1.ScheduleEntryLockScheduleKind.YearDay;
            const schedule = (self.state.get(StateKeys.schedule(userId, slotId, kind)) ?? {});
            const cc = new ScheduleEntryLockCC_1.ScheduleEntryLockCCYearDayScheduleReport(self.host, {
                nodeId: controller.host.ownNodeId,
                userId,
                slotId,
                ...schedule,
            });
            return { action: "sendCC", cc };
        }
    },
};
const respondToScheduleEntryLockDailyRepeatingScheduleSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC
            instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCDailyRepeatingScheduleSet) {
            const userCodeCapabilities = {
                ...UserCode_1.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            // If the user identifier is out of range, the command will be ignored
            const userId = receivedCC.userId;
            if (userId > userCodeCapabilities.numUsers) {
                return { action: "fail" };
            }
            const capabilities = {
                ...defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["Schedule Entry Lock"], receivedCC.endpointIndex),
            };
            const slotId = receivedCC.slotId;
            // Ignore out of range slot queries
            if (slotId > capabilities.numDailyRepeatingSlots) {
                return { action: "fail" };
            }
            const kind = cc_1.ScheduleEntryLockScheduleKind.DailyRepeating;
            const schedule = receivedCC.action === cc_1.ScheduleEntryLockSetAction.Set
                ? {
                    weekdays: receivedCC.weekdays,
                    startHour: receivedCC.startHour,
                    startMinute: receivedCC.startMinute,
                    durationHour: receivedCC.durationHour,
                    durationMinute: receivedCC.durationMinute,
                }
                : undefined;
            self.state.set(StateKeys.schedule(userId, slotId, kind), schedule);
            return { action: "ok" };
        }
    },
};
const respondToScheduleEntryLockDailyRepeatingScheduleGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC
            instanceof ScheduleEntryLockCC_1.ScheduleEntryLockCCDailyRepeatingScheduleGet) {
            const userCodeCapabilities = {
                ...UserCode_1.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            // If the user identifier is out of range, the command will be ignored
            const userId = receivedCC.userId;
            if (userId > userCodeCapabilities.numUsers) {
                return { action: "fail" };
            }
            const capabilities = {
                ...defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["Schedule Entry Lock"], receivedCC.endpointIndex),
            };
            const slotId = receivedCC.slotId;
            // Ignore out of range slot queries
            if (slotId > capabilities.numDailyRepeatingSlots) {
                return { action: "fail" };
            }
            const kind = cc_1.ScheduleEntryLockScheduleKind.DailyRepeating;
            const schedule = (self.state.get(StateKeys.schedule(userId, slotId, kind)) ?? {});
            const cc = new ScheduleEntryLockCC_1.ScheduleEntryLockCCDailyRepeatingScheduleReport(self.host, {
                nodeId: controller.host.ownNodeId,
                userId,
                slotId,
                ...schedule,
            });
            return { action: "sendCC", cc };
        }
    },
};
exports.ScheduleEntryLockCCBehaviors = [
    respondToScheduleEntryLockSupportedGet,
    respondToScheduleEntryLockTimeOffsetSet,
    respondToScheduleEntryLockTimeOffsetGet,
    respondToScheduleEntryLockEnableSet,
    respondToScheduleEntryLockEnableAllSet,
    respondToScheduleEntryLockWeekDayScheduleSet,
    respondToScheduleEntryLockWeekDayScheduleGet,
    respondToScheduleEntryLockYearDayScheduleSet,
    respondToScheduleEntryLockYearDayScheduleGet,
    respondToScheduleEntryLockDailyRepeatingScheduleSet,
    respondToScheduleEntryLockDailyRepeatingScheduleGet,
];
//# sourceMappingURL=ScheduleEntryLock.js.map