"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserCodeCCBehaviors = exports.defaultCapabilities = void 0;
const cc_1 = require("@zwave-js/cc");
const UserCodeCC_1 = require("@zwave-js/cc/UserCodeCC");
const safe_1 = require("@zwave-js/core/safe");
exports.defaultCapabilities = {
    numUsers: 1,
    supportedASCIIChars: "0123456789",
    supportsAdminCode: true,
    supportsAdminCodeDeactivation: true,
    supportsUserCodeChecksum: true,
    supportedKeypadModes: [cc_1.KeypadMode.Normal],
    supportedUserIDStatuses: [
        cc_1.UserIDStatus.Available,
        cc_1.UserIDStatus.Enabled,
        cc_1.UserIDStatus.Disabled,
    ],
};
const STATE_KEY_PREFIX = "UserCode_";
const StateKeys = {
    userCode: (userId) => `${STATE_KEY_PREFIX}userCode_${userId}`,
    userIdStatus: (userId) => `${STATE_KEY_PREFIX}userIdStatus_${userId}`,
    adminCode: `${STATE_KEY_PREFIX}adminCode`,
    keypadMode: `${STATE_KEY_PREFIX}keypadMode`,
};
const respondToUsersNumberGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCUsersNumberGet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            const cc = new UserCodeCC_1.UserCodeCCUsersNumberReport(self.host, {
                nodeId: controller.host.ownNodeId,
                supportedUsers: capabilities.numUsers ?? 1,
            });
            return { action: "sendCC", cc };
        }
    },
};
const respondToUserGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCGet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            const userId = receivedCC.userId;
            let cc;
            if (capabilities.numUsers >= userId) {
                cc = new UserCodeCC_1.UserCodeCCReport(self.host, {
                    nodeId: controller.host.ownNodeId,
                    userId,
                    userIdStatus: (self.state.get(StateKeys.userIdStatus(userId)) ?? cc_1.UserIDStatus.Available),
                    userCode: self.state.get(StateKeys.userCode(userId)),
                });
            }
            else {
                cc = new UserCodeCC_1.UserCodeCCReport(self.host, {
                    nodeId: controller.host.ownNodeId,
                    userId,
                    userIdStatus: cc_1.UserIDStatus.StatusNotAvailable,
                });
            }
            return { action: "sendCC", cc };
        }
    },
};
const respondToUserCodeSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCSet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            const userId = receivedCC.userId;
            const userIdStatus = receivedCC.userIdStatus;
            if (capabilities.numUsers >= userId) {
                self.state.set(StateKeys.userIdStatus(userId), userIdStatus);
                const code = userIdStatus !== cc_1.UserIDStatus.Available
                    && userIdStatus !== cc_1.UserIDStatus.StatusNotAvailable
                    ? receivedCC.userCode
                    : undefined;
                self.state.set(StateKeys.userCode(userId), code);
                return { action: "ok" };
            }
            return { action: "fail" };
        }
    },
};
const respondToUserCodeCapabilitiesGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCCapabilitiesGet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            const cc = new UserCodeCC_1.UserCodeCCCapabilitiesReport(self.host, {
                nodeId: controller.host.ownNodeId,
                supportsAdminCode: capabilities.supportsAdminCode,
                supportsAdminCodeDeactivation: capabilities
                    .supportsAdminCodeDeactivation,
                supportsUserCodeChecksum: capabilities
                    .supportsUserCodeChecksum,
                supportsMultipleUserCodeReport: false,
                supportsMultipleUserCodeSet: false,
                supportedUserIDStatuses: capabilities.supportedUserIDStatuses,
                supportedKeypadModes: capabilities.supportedKeypadModes,
                supportedASCIIChars: capabilities.supportedASCIIChars,
            });
            return { action: "sendCC", cc };
        }
    },
};
const respondToUserCodeKeypadModeGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCKeypadModeGet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            const cc = new UserCodeCC_1.UserCodeCCKeypadModeReport(self.host, {
                nodeId: controller.host.ownNodeId,
                keypadMode: (self.state.get(StateKeys.keypadMode)
                    ?? capabilities.supportedKeypadModes?.[0]
                    ?? cc_1.KeypadMode.Normal),
            });
            return { action: "sendCC", cc };
        }
    },
};
const respondToUserCodeKeypadModeSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCKeypadModeSet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            if (capabilities.supportedKeypadModes?.includes(receivedCC.keypadMode)) {
                self.state.set(StateKeys.keypadMode, receivedCC.keypadMode);
                return { action: "ok" };
            }
            return { action: "fail" };
        }
    },
};
const respondToUserCodeAdminCodeSet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCAdminCodeSet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            const adminCode = receivedCC.adminCode;
            if (capabilities.supportsAdminCode) {
                if (adminCode.length > 0
                    || capabilities.supportsAdminCodeDeactivation) {
                    self.state.set(StateKeys.adminCode, receivedCC.adminCode);
                    return { action: "ok" };
                }
            }
            return { action: "fail" };
        }
    },
};
const respondToUserCodeAdminCodeGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCAdminCodeGet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            let adminCode;
            if (capabilities.supportsAdminCode) {
                adminCode = self.state.get(StateKeys.adminCode);
            }
            const cc = new UserCodeCC_1.UserCodeCCAdminCodeReport(self.host, {
                nodeId: controller.host.ownNodeId,
                adminCode: adminCode ?? "",
            });
            return { action: "sendCC", cc };
        }
    },
};
const respondToUserCodeUserCodeChecksumGet = {
    handleCC(controller, self, receivedCC) {
        if (receivedCC instanceof UserCodeCC_1.UserCodeCCUserCodeChecksumGet) {
            const capabilities = {
                ...exports.defaultCapabilities,
                ...self.getCCCapabilities(safe_1.CommandClasses["User Code"], receivedCC.endpointIndex),
            };
            if (capabilities.supportsUserCodeChecksum) {
                let data = Buffer.allocUnsafe(0);
                for (let i = 1; i <= capabilities.numUsers; i++) {
                    const status = self.state.get(StateKeys.userIdStatus(i));
                    let code = (self.state.get(StateKeys.userCode(i)) ?? "");
                    if (status === undefined
                        || status === cc_1.UserIDStatus.Available
                        || code.length === 0) {
                        continue;
                    }
                    const tmp = Buffer.allocUnsafe(3 + code.length);
                    tmp.writeUInt16BE(i, 0);
                    tmp[2] = status;
                    if (typeof code === "string") {
                        code = Buffer.from(code, "ascii");
                    }
                    code.copy(tmp, 3);
                    data = Buffer.concat([data, tmp]);
                }
                const checksum = data.length > 0 ? (0, safe_1.CRC16_CCITT)(data) : 0x0000;
                const cc = new UserCodeCC_1.UserCodeCCUserCodeChecksumReport(self.host, {
                    nodeId: controller.host.ownNodeId,
                    userCodeChecksum: checksum,
                });
                return { action: "sendCC", cc };
            }
            return { action: "stop" };
        }
    },
};
exports.UserCodeCCBehaviors = [
    respondToUsersNumberGet,
    respondToUserGet,
    respondToUserCodeSet,
    respondToUserCodeCapabilitiesGet,
    respondToUserCodeKeypadModeGet,
    respondToUserCodeKeypadModeSet,
    respondToUserCodeAdminCodeGet,
    respondToUserCodeAdminCodeSet,
    respondToUserCodeUserCodeChecksumGet,
];
//# sourceMappingURL=UserCode.js.map