"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigManager = void 0;
const core_1 = require("@zwave-js/core");
const shared_1 = require("@zwave-js/shared");
const fs_extra_1 = require("fs-extra");
const node_path_1 = __importDefault(require("node:path"));
const Logger_1 = require("./Logger");
const Manufacturers_1 = require("./Manufacturers");
const DeviceConfig_1 = require("./devices/DeviceConfig");
const utils_1 = require("./utils");
class ConfigManager {
    constructor(options = {}) {
        this.logger = new Logger_1.ConfigLogger(options.logContainer ?? new core_1.ZWaveLogContainer({ enabled: false }));
        this.deviceConfigPriorityDir = options.deviceConfigPriorityDir;
        this._configVersion =
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            require("@zwave-js/config/package.json").version;
    }
    _configVersion;
    get configVersion() {
        return this._configVersion;
    }
    logger;
    _manufacturers;
    get manufacturers() {
        if (!this._manufacturers) {
            throw new core_1.ZWaveError("The config has not been loaded yet!", core_1.ZWaveErrorCodes.Driver_NotReady);
        }
        return this._manufacturers;
    }
    deviceConfigPriorityDir;
    index;
    fulltextIndex;
    _useExternalConfig = false;
    get useExternalConfig() {
        return this._useExternalConfig;
    }
    async loadAll() {
        // If the environment option for an external config dir is set
        // try to sync it and then use it
        const syncResult = await (0, utils_1.syncExternalConfigDir)(this.logger);
        if (syncResult.success) {
            this._useExternalConfig = true;
            this.logger.print(`Using external configuration dir ${(0, utils_1.externalConfigDir)()}`);
            this._configVersion = syncResult.version;
        }
        else {
            this._useExternalConfig = false;
            this._configVersion = await (0, utils_1.getEmbeddedConfigVersion)();
        }
        this.logger.print(`version ${this._configVersion}`, "info");
        await this.loadManufacturers();
        await this.loadDeviceIndex();
    }
    async loadManufacturers() {
        try {
            this._manufacturers = await (0, Manufacturers_1.loadManufacturersInternal)(this._useExternalConfig);
        }
        catch (e) {
            // If the config file is missing or invalid, don't try to find it again
            if ((0, core_1.isZWaveError)(e) && e.code === core_1.ZWaveErrorCodes.Config_Invalid) {
                if (process.env.NODE_ENV !== "test") {
                    this.logger.print(`Could not load manufacturers config: ${e.message}`, "error");
                }
                if (!this._manufacturers)
                    this._manufacturers = new Map();
            }
            else {
                // This is an unexpected error
                throw e;
            }
        }
    }
    async saveManufacturers() {
        if (!this._manufacturers) {
            throw new core_1.ZWaveError("The config has not been loaded yet!", core_1.ZWaveErrorCodes.Driver_NotReady);
        }
        await (0, Manufacturers_1.saveManufacturersInternal)(this._manufacturers);
    }
    /**
     * Looks up the name of the manufacturer with the given ID in the configuration DB
     * @param manufacturerId The manufacturer id to look up
     */
    lookupManufacturer(manufacturerId) {
        if (!this._manufacturers) {
            throw new core_1.ZWaveError("The config has not been loaded yet!", core_1.ZWaveErrorCodes.Driver_NotReady);
        }
        return this._manufacturers.get(manufacturerId);
    }
    /**
     * Add new manufacturers to configuration DB
     * @param manufacturerId The manufacturer id to look up
     * @param manufacturerName The manufacturer name
     */
    setManufacturer(manufacturerId, manufacturerName) {
        if (!this._manufacturers) {
            throw new core_1.ZWaveError("The config has not been loaded yet!", core_1.ZWaveErrorCodes.Driver_NotReady);
        }
        this._manufacturers.set(manufacturerId, manufacturerName);
    }
    async loadDeviceIndex() {
        try {
            // The index of config files included in this package
            const embeddedIndex = await (0, DeviceConfig_1.loadDeviceIndexInternal)(this.logger, this._useExternalConfig);
            // A dynamic index of the user-defined priority device config files
            const priorityIndex = [];
            if (this.deviceConfigPriorityDir) {
                if (await (0, fs_extra_1.pathExists)(this.deviceConfigPriorityDir)) {
                    priorityIndex.push(...(await (0, DeviceConfig_1.generatePriorityDeviceIndex)(this.deviceConfigPriorityDir, this.logger)));
                }
                else {
                    this.logger.print(`Priority device configuration directory ${this.deviceConfigPriorityDir} not found`, "warn");
                }
            }
            // Put the priority index in front, so the files get resolved first
            this.index = [...priorityIndex, ...embeddedIndex];
        }
        catch (e) {
            // If the index file is missing or invalid, don't try to find it again
            if ((!(0, core_1.isZWaveError)(e) && e instanceof Error)
                || ((0, core_1.isZWaveError)(e)
                    && e.code === core_1.ZWaveErrorCodes.Config_Invalid)) {
                // Fall back to no index on production systems
                if (!this.index)
                    this.index = [];
                if (process.env.NODE_ENV !== "test") {
                    this.logger.print(`Could not load or regenerate device config index: ${e.message}`, "error");
                    // and don't throw
                    return;
                }
                // But fail hard in tests
                throw e;
            }
        }
    }
    getIndex() {
        return this.index;
    }
    async loadFulltextDeviceIndex() {
        this.fulltextIndex = await (0, DeviceConfig_1.loadFulltextDeviceIndexInternal)(this.logger);
    }
    getFulltextIndex() {
        return this.fulltextIndex;
    }
    /**
     * Looks up the definition of a given device in the configuration DB, but does not evaluate conditional settings.
     * @param manufacturerId The manufacturer id of the device
     * @param productType The product type of the device
     * @param productId The product id of the device
     * @param firmwareVersion If known, configuration for a specific firmware version can be loaded.
     * If this is `undefined` or not given, the first matching file with a defined firmware range will be returned.
     */
    async lookupDevicePreserveConditions(manufacturerId, productType, productId, firmwareVersion) {
        // Load/regenerate the index if necessary
        if (!this.index)
            await this.loadDeviceIndex();
        // Look up the device in the index
        const indexEntries = this.index.filter((0, utils_1.getDeviceEntryPredicate)(manufacturerId, productType, productId, firmwareVersion));
        // If there are multiple with overlapping firmware ranges, return the preferred one first
        const indexEntry = indexEntries.find((e) => !!e.preferred)
            ?? indexEntries[0];
        if (indexEntry) {
            const devicesDir = (0, DeviceConfig_1.getDevicesPaths)(this._useExternalConfig ? (0, utils_1.externalConfigDir)() : utils_1.configDir).devicesDir;
            const filePath = node_path_1.default.isAbsolute(indexEntry.filename)
                ? indexEntry.filename
                : node_path_1.default.join(devicesDir, indexEntry.filename);
            if (!(await (0, fs_extra_1.pathExists)(filePath)))
                return;
            // A config file is treated as am embedded one when it is located under the devices root dir
            // or the external config dir
            const isEmbedded = !node_path_1.default
                .relative(devicesDir, filePath)
                .startsWith("..");
            // When a device file is located in a different root directory than the embedded config files,
            // we use the embedded dir a fallback
            const rootDir = indexEntry.rootDir ?? devicesDir;
            const fallbackDirs = rootDir === devicesDir
                ? undefined
                : [devicesDir];
            try {
                return await DeviceConfig_1.ConditionalDeviceConfig.from(filePath, isEmbedded, { rootDir, fallbackDirs });
            }
            catch (e) {
                if (process.env.NODE_ENV !== "test") {
                    this.logger.print(`Error loading device config ${filePath}: ${(0, shared_1.getErrorMessage)(e, true)}`, "error");
                }
            }
        }
    }
    /**
     * Looks up the definition of a given device in the configuration DB
     * @param manufacturerId The manufacturer id of the device
     * @param productType The product type of the device
     * @param productId The product id of the device
     * @param firmwareVersion If known, configuration for a specific firmware version can be loaded.
     * If this is `undefined` or not given, the first matching file with a defined firmware range will be returned.
     */
    async lookupDevice(manufacturerId, productType, productId, firmwareVersion) {
        const ret = await this.lookupDevicePreserveConditions(manufacturerId, productType, productId, firmwareVersion);
        return ret?.evaluate({
            manufacturerId,
            productType,
            productId,
            firmwareVersion,
        });
    }
}
exports.ConfigManager = ConfigManager;
//# sourceMappingURL=ConfigManager.js.map